# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Copyright (c) 2007 by Intevation GmbH
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
#

import logging

from waskaweb.lib.base                      import *
from waskaweb.controllers.CaseBase          import CasebaseController 
from waskaweb.model.repeatgroup             import AidList 
from waskaweb.model.agency                  import *
from waskaweb.model.case                    import *
from waskaweb.model.statement               import * 
from waskaweb.lib.security                  import checkRole
from waskaweb.lib.search                    import INCONSISTENCY_CHECK_AFTER
from waskaweb.model.phase_transition        import PHASE_DESCRIPTIONS

from waskaweb.controllers.case              import ANONYMIZE_CONFIRM, \
                                                   MARKANONYMIZE_CONFIRM_TEXT, \
                                                   MARKANONYMIZE_SUCCESS, \
                                                   MARKANONYMIZE_SUCCESS_TEXT, \
                                    MARKANONYMIZE_INCOMPLETE_PHASE_FAILED, \
                                    MARKANONYMIZE_INCOMPLETE_PHASE_FAILED_TEXT,\
                                    MARKANONYMIZE_CLEARING_PHASE_FAILED_TEXT, \
                                    _determineAnonymizedCaseEndPhase

log = logging.getLogger(__name__)

HEADER           = u"""Einverständniserklärung setzen?""" 
ACCEPT_QUESTION  = u"""Sie haben angegeben, dass eine Einverständniserklärung des Jugendlichen zur Erfassung, Verarbeitung und Speicherung personenbezogener Daten vorliegt. Sind Sie sicher das diese Angabe korrekt ist?"""  
DECLINE_QUESTION = u"""Sie haben angegeben, dass keine Einverständniserklärung des Jugendlichen zur Erfassung, Verarbeitung und Speicherung personenbezogener Daten vorliegt, oder Sie eine bestehende Einverständniserklärung zurückziehen wollen. Dies bedeutet das die personenbezogenen Daten der Fallakte zur Wahrung des Datenschutzes anonymisiert werden und unwiederbringlich verloren gehen. Die Fallakte ist danach nicht mehr bearbeitbar. Sind Sie sicher das diese Angabe korrekt ist?"""  
SET_EE_ACCEPT_SUCCESS_HEADER = u"""Einwilligungserklärung gesetzt!"""
SET_EE_ACCEPT_SUCCESS_TEXT = u"""Bitte klicken Sie auf "OK", um mit der Bearbeitung der Fallakte fortzufahren."""
SET_EE_ACCEPT_FAILED_HEADER = u"""Einwilligungserklärung nicht gesetzt!"""
SET_EE_ACCEPT_FAILED_TEXT = u"""Bei dem Setzen der Einwilligunserklärung ist ein Fehler aufgetreten. Bitte klicken Sie auf "OK", um fortzufahren."""
#SET_EE_DECLINE_SUCCESS_HEADER = u"""Fallakte wurde anonymisiert!"""
#SET_EE_DECLINE_SUCCESS_TEXT = u"""Die personenbezogenen Daten der Fallakte wurden gelöscht und die Fallakte aus der Übersicht entfernt. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""
SET_EE_DECLINE_FAILED_HEADER = u"""Fallakte wurde nicht anonymisiert!"""
SET_EE_DECLINE_FAILED_TEXT = u"""Bei dem Anonymisieren der Fallakte ist ein Fehler aufgetreten. Die Fallakte wurde nicht anonymisiert. Bitte klicken Sie auf "OK", um zur Fallaktenübersicht fortzufahren."""
MARK_EE_DECLINE_INCOMPLETE_PHASE_FAILED = u"""Die Einwilligungserklärung wurde nicht zurückgezogen!"""
MARK_EE_DECLINE_INCOMPLETE_PHASE_FAILED_TEXT = u"""<p>Das Zurückziehen der Einwilligungserklärung führt zur Anonymisierung der Akte.</p>%s""" % MARKANONYMIZE_INCOMPLETE_PHASE_FAILED_TEXT
MARK_EE_DECLINE_CLEARING_PHASE_FAILED_TEXT = u"""<p>Das Zurückziehen der Einwilligungserklärung führt zur Anonymisierung der Akte.</p>%s""" % MARKANONYMIZE_CLEARING_PHASE_FAILED_TEXT

class StatementController(CasebaseController):

    def __before__(self):
        CasebaseController.__before__(self)

    @checkRole('cm_ka')
    def index(self):
        redirect_to(controller="waska", action="start")

    @checkRole('cm_ka')
    def listStatements(self, id):
        c.form_navigation = self._getFormNavigation()
        return render('statement/list_statements.mako')

    @checkRole('cm_ka')
    def printAidPlanStatement(self, id):
        id        = self._checkInt(id)
        case      = self._loadCase(id)
        statement = case.getAidPlanStatement()
        c.content = statement.getContent() 
        return render('statement/default_statement.mako')

    @checkRole('cm_ka')
    def printPrivacyStatement(self, id):
        id        = self._checkInt(id)
        case      = self._loadCase(id)
        statement = case.getPrivacyStatement() 
        c.content = statement.getContent()
        return render('statement/default_statement.mako')

    @checkRole('cm_ka')
    def printDiscretionStatement(self, id):
        id        = self._checkInt(id)
        case      = self._loadCase(id)
        statement = case.getDiscretionStatement() 
        c.content = statement.getContent()
        return render('statement/default_statement.mako')

    @checkRole('cm_ka')
    def setPrivacyStatement(self, id):
        id                = self._checkInt(id)
        case              = self._loadCase(id)
        c.form_navigation = self._getFormNavigation()
        c.ds_id           = id
        return render('statement/edit.mako')


    @checkRole('cm_ka')
    def acceptPrivacyStatement(self, id, confirmed):
        id                = self._checkInt(id)
        confirmed         = self._checkBool(confirmed)
        case              = self._loadCase(id)
        c.form_navigation = self._getFormNavigation()
        if confirmed:
            try:
                statement = case.getPrivacyStatement()
                statement.sign()
                # TODO: Think about refreshing the state of the privacy statement
                # in the statement model
                session_case                   = session.get('case')
                session_case.privacy_statement = True
                session.save()
                c.success_for                  = SET_EE_ACCEPT_SUCCESS_HEADER
                c.success_text                 = SET_EE_ACCEPT_SUCCESS_TEXT
                c.url_ok                       = h.url_for(controller="/case", action="digest", id=case.id)
                return render('statement/dialogs/ee_accept_success.mako')
            except:
                c.failed_for   = SET_EE_ACCEPT_FAILED_HEADER
                c.failed_text  = SET_EE_ACCEPT_FAILED_TEXT
                c.url_ok       = h.url_for(controller='/statement', action='setPrivacyStatement', id=case.id)
                return render('/statement/dialogs/ee_accept_failed.mako')
        else:
            c.confirm_for = HEADER
            c.question = ACCEPT_QUESTION
            c.url_yes = h.url_for(
                controller="statement", \
                action="acceptPrivacyStatement", \
                id=id, \
                confirmed="1")
            c.url_no = h.url_for(controller="/case", \
                action="select", \
                id=id,
                confirmed=0)
            return render('statement/dialogs/confirm.mako')

    @checkRole('cm_ka')
    def declinePrivacyStatement(self, id, confirmed):
        id                = self._checkInt(id)
        confirmed         = self._checkBool(confirmed)
        case              = self._loadCase(id)
        c.form_navigation = self._getFormNavigation()
        if (not case.getState().phasesAreConsistent() or not case.getState().phasesAreCompleted()) and case.isYoungerThan(INCONSISTENCY_CHECK_AFTER):
            c.notification_for  = MARK_EE_DECLINE_INCOMPLETE_PHASE_FAILED
            if case.getState().getPhase() in (CLEAR_START, CLEAR_ENDE):
                c.notification_text = \
                            MARK_EE_DECLINE_CLEARING_PHASE_FAILED_TEXT
            else:    
                c.notification_text = \
                            MARK_EE_DECLINE_INCOMPLETE_PHASE_FAILED_TEXT
            c.url_ok            = "/case/digest/%s" % id
            return render('/casemanagement/dialogs/failed_markanonymize_phase_incomplete.mako')
        if confirmed:
            try:
                statement = case.getPrivacyStatement()
                statement.decline()
                # Mark case for anonymizion
                case.getState().setState(4)

                # TODO: Think about refreshing the state of the privacy statement
                # in the statement model
                session_case = session.get('case')
                session_case.privacy_statement = True
                session.save()
                c.success_for  = MARKANONYMIZE_SUCCESS
                c.success_text = MARKANONYMIZE_SUCCESS_TEXT
                c.url_ok       = h.url_for ('/case_overview')
                return render('/statement/dialogs/ee_decline_success.mako')
            except:
                c.failed_for   = SET_EE_DECLINE_FAILED_HEADER
                c.failed_text  = SET_EE_DECLINE_FAILED_TEXT
                c.url_ok       = h.url_for ('/case_overview')
                return render('/statement/dialogs/ee_decline_failed.mako')
        else:
            c.confirm_for = ANONYMIZE_CONFIRM
            c.question = MARKANONYMIZE_CONFIRM_TEXT % \
                      PHASE_DESCRIPTIONS[_determineAnonymizedCaseEndPhase(case)]
            c.url_yes = h.url_for(
                controller="statement", \
                action="declinePrivacyStatement", \
                id=id, \
                confirmed="1")
            c.url_no = h.url_for(controller="/case", \
                action="select", \
                id=id,
                confirmed=0)
            return render('statement/dialogs/confirm.mako')

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8:
