# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from types import StringTypes

from sql_helper import range_compress, where_ids

import sys

from exportselection import SelectorFactory, CasePartsSelection, RG_ALL

SEP = "\t"

FETCH_ALL_IDS = \
"""SELECT id FROM master_tbl_view"""

BLACK_LIST = set(['id', 'master_id', 'uuid_id', 'bearbeiter_id'])

def strReplace(x):
    if type(x) in StringTypes:
        x = unicode(str(x), "utf-8")
        return x.replace("\r", "").replace("\n", " ").replace(SEP, "    ")
    if x is None: return ""
    return x

class RepeatGroup:

    def __init__(self):
        self.headers = None
        self.values  = None

    def headerFields(self):
        return self.headers

    def valueFields(self):
        return self.values

    def numFields(self):
        return len(self.values)

    def setColumns(self, headers, values):
        self.headers = headers
        self.values  = values

class Case(RepeatGroup):

    def __init__(self, orig_pos):
        RepeatGroup.__init__(self)
        self.orig_pos = orig_pos
        self.rgs = [[] for i in range(len(RG_ALL))]
        
    def appendRepeatGroup(self, rg, index):
        self.rgs[index].append(rg)

class Cases:
    def __init__(self, anonymize=False):
        self.cases     = None
        self.anonymize = anonymize

    def fetchCases(self, cur, ids, selector_factory):

        case_hash = dict([(x[1], Case(x[0])) for x in enumerate(ids)])
        ids.sort()
        ids = range_compress(ids)

        master_where = where_ids(ids)
        rg_where     = where_ids(ids, 'master_id')

        selector = selector_factory.createSelector(master_where, rg_where)

        stmnt = selector.getSelectStatementByView('master_tbl')

        #print >> sys.stderr, "stmnt: %s" % stmnt

        cur.execute(stmnt)

        headers, used_indices = None, None

        while True:
            row = cur.fetchone()
            if not row: break

            if used_indices is None:
                used_indices = [t[0] for t in enumerate(cur.description) if t[1][0] not in BLACK_LIST]
                headers      = [cur.description[i][0] for i in used_indices]

            values = [row[i] for i in used_indices]

            case_hash[row[0]].setColumns(headers, values)

        for index, view in enumerate(RG_ALL):

            headers, used_indices = None, None

            stmnt = selector.getSelectStatementByView(view)

            if not stmnt: continue

            cur.execute(stmnt)

            while True:
                row = cur.fetchone()
                if not row: break
                if used_indices is None:
                    used_indices = [t[0] for t in enumerate(cur.description) if t[1][0] not in BLACK_LIST]
                    headers      = [cur.description[i][0] for i in used_indices]

                values = [row[i] for i in used_indices]

                rg = RepeatGroup()
                rg.setColumns(headers, values)
                case_hash[row[1]].appendRepeatGroup(rg, index)

        self.cases = case_hash.values()
        self.cases.sort(lambda a, b: cmp(a.orig_pos, b.orig_pos))

    def maxRepeatGroups(self):
        mc = [0] * len(RG_ALL)
        for rgs in [c.rgs for c in self.cases]:
            for i in range(len(mc)):
                mc[i] = max(mc[i], len(rgs[i]))
        self.maxRGs = mc

    def noneEmptyRepeatGroup(self, i):
        for case in self.cases:
            rgs = case.rgs
            if len(rgs[i]):
                return rgs[i][0]
        return None

    def generateHeader(self, out):
        if not self.cases:
            return ""
        case = self.cases[0]
        headers = case.headerFields()
        for i in range(len(RG_ALL)):
            rg = self.noneEmptyRepeatGroup(i)
            if rg is None:
                continue
            hs = rg.headerFields() 
            for j in range(self.maxRGs[i]):
                if j > 0:
                    headers.extend([ "%s$%d" % (x, j) for x in hs])
                else:
                    headers.extend(hs)
        print >> out, "#%s" % SEP.join(headers)

    def dumpFields(self, out):
        fills = []
        for i in range(len(RG_ALL)):
            rg = self.noneEmptyRepeatGroup(i)
            if rg is None: c = 0
            else:          c = rg.numFields()
            fills.append([''] * c)
            
        for case in self.cases:
            line = case.valueFields()
            for m, rg, fill in zip(self.maxRGs, case.rgs, fills):
                count = 0
                for r in rg:
                    line.extend(r.valueFields())
                    count += 1
                while count < m:
                    line.extend(fill)
                    count += 1
            print >> out, "%s" % SEP.join(map(lambda x: u"%s" % strReplace(x), line))


def exportAsCSV(cur, out, ids=None, case_parts=None, anonymize=False):
    #from datetime import datetime
    #start = datetime.now()
    cases = Cases(anonymize)
    if ids is None:
        cur.execute(FETCH_ALL_IDS)
        ids = []
        while True:
            row = cur.fetchone()
            if row is None: break
            ids.append(int(row[0]))

    if case_parts is None:
        case_parts = CasePartsSelection()

    selector_factory = SelectorFactory(case_parts, anonymize)

    cases.fetchCases(cur, ids, selector_factory)

    cases.maxRepeatGroups()
    cases.generateHeader(out)
    cases.dumpFields(out)

    #stop = datetime.now()
    #print >> sys.stderr, "export took: %s" % (stop-start)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
