# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#
from paste.registry import StackedObjectProxy

import psycopg2 as dbapi

import sys
import traceback

db = StackedObjectProxy(name="waska.db")

def enter(dbObject):
    db._push_object(dbObject)

def leave(dbObject=None):
    db._pop_object(dbObject)

class DB:
    def __init__(self, dbname, user, password, host='localhost', port=5432):
        self.dbname      = dbname 
        self.user        = user
        self.password    = password.replace("\\", "\\\\")
        self.host        = host
        self.port        = int(port)
        self.conn        = None
        self.connections = []

    def getConnection(self):
        for used in self.connections:
            if not used[0]:
                used[0] = True
                return used[1]

        con = dbapi.connect(
            database = self.dbname,
            host     = self.host,
            port     = self.port,
            user     = self.user,
            password = self.password)
        used = [True, con]
        self.connections.append(used)
        return con

    def recycleConnection(self, con, cur=None):
        if cur:
            try: cur.close()
            except: pass
        if con:
            for used in self.connections:
                if used[1] == con:
                    used[0] = False
                    return
            print >> sys.stderr, "WARNING: tried to recycle unused connection"

    def closeConnections(self):
        cons = self.connections
        self.connections = []
        for used in cons:
            if used[0]:
                print >> sys.stderr, "WARNING: Closing an unrecylcled connection"
            try:
                used[0].close()
            except:
                pass

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
