# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
# Torsten Irlaender <torsten.irlaender@intevation.de>
#

from waskaweb.lib.db import DB, db, enter, leave
from pylons          import request, session
import re
import traceback
import sys
import time
import os
import random
import md5

from codecs import getdecoder

import psycopg2 as dbapi

from waskaweb.lib.helpers import hasRole
from decorator            import decorator
from paste.httpexceptions import HTTPUnauthorized

from types import ListType, TupleType

HOST   = '192.168.11.17'
PORT   = 5433
DBNAME = u'ka_%s_db'
USER   = u'ka_%s_%s'

# maps new FKZs to old
NEW_DBS = {}

check_re            = re.compile(r"^[a-z_0-9]+$")
ROLE_SQL            = """select groname from pg_group where getOID() = ANY (grolist)"""
ROLE_SQL            = """select where getOID() = ANY (grolist)"""
FETCH_USER_DATA_SQL = """SELECT id, vorname, nachname, passwort_aendern, aktiviert, rolle, gid, last_login from ka_benutzer_tbl_view WHERE login = %(login)s"""
UPDATE_LASTLOGIN_SQL = """SELECT set_last_logintime(%(user_id)s)"""
UNAUTHORIZED_DETAIL = \
"""Sie sind nicht authorisiert, das angeforderte Dokument anzuschauen.
"""

ASCII_DECODER = getdecoder("ascii")

def validPassword(s):
    try:
        ASCII_DECODER(s)
    except (UnicodeEncodeError, UnicodeDecodeError):
        return False
    return True

def checkLogin(user, password):

    dbname = getDbName()
    if not password                \
    or not validPassword(password) \
    or not dbname                  \
    or not check_re.match(user)    \
    or not check_re.match(dbname):
        return None

    try:
        mydb = DB(
            dbname   = DBNAME % dbname,
            host     = HOST,
            port     = PORT,
            user     = USER % (dbname, user),
            password = password)

        try:
            enter(mydb)
            conn, cursor = None, None
            try:
                conn = db.getConnection()
                cursor = conn.cursor()
                login = "ka_%s_%s" % (dbname, user)
                fields = {'login': login}
                cursor.execute(FETCH_USER_DATA_SQL, fields)
                try:
                    result = cursor.fetchone()
                    data = {}
                    data['id']         = result[0] 
                    data['firstname']  = unicode(str(result[1]), 'utf-8')
                    data['lastname']   = unicode(str(result[2]), 'utf-8')
                    data['newpass']    = result[3]
                    data['activated']  = result[4] 
                    data['role']       = unicode(str(result[5]), 'utf-8')
                    data['gid']        = result[6]
                    data['last_login'] = result[7]
                except:
                    data = None 
            finally:
                db.recycleConnection(conn, cursor)

            try:
                from waskaweb.model.user import SessionUser, SessionSuperAdmin 
                if data:
                    userobject = SessionUser(login, password)
                    userobject.setData(data)
                elif user == 'adm':
                    userobject = SessionSuperAdmin(login, password)
                else:
                    return None

                # UPDATE last login time
                conn, cursor = None, None
                try:
                    conn   = db.getConnection()
                    cursor = conn.cursor()
                    if data:
                        try:
                            fields = {'user_id': data.get('id')}
                            cursor.execute(UPDATE_LASTLOGIN_SQL, fields)
                            conn.commit()
                        except StandardError, err:
                            traceback.print_exc(file=sys.stderr)
                            print "Error on setting last logintime: %s" % cursor.query 
                finally:
                    db.recycleConnection(conn, cursor)

            except StandardError, err:
                traceback.print_exc(file=sys.stderr)
                print "Error on creating user Object: %s" % err

            #RETURN userobject
            return userobject
        finally:
            leave(mydb)
            mydb.closeConnections()
    except dbapi.OperationalError, err:
        # This is likely a login failure
        # XXX: Is there a way in DB Api 2.0 to determine
        # this database independent?
        pass
    except: 
        traceback.print_exc(file=sys.stderr)
    return None

def generateID(obj=None):
    array = [time.time(), os.times(), random.random()]
    if not obj is None: array.append(id(obj))
    return md5.new(str(array)).hexdigest()

def subnet(addr):
    '''poor man's subnet splitting'''
    x = addr.rsplit('.', 1) # IPv4
    if len(x) == 2: return x[0]
    x = addr.rsplit(':', 1) # IPv6
    if len(x) == 2: return x[0]
    return addr

def userIdentity():
    env = request.environ
    return "%s" % env.get('HTTP_USER_AGENT', '')

def load_db_mapping_from_file(fname):
    if os.path.isfile(fname):
        global NEW_DBS
        f = None
        try:
            f = open(fname, "r")
            for line in f:
                line = line.strip()
                if not line or line.startswith("#"): continue
                parts = line.split(';')
                if len(parts) > 1:
                    NEW_DBS[parts[0]] = parts[1]
        finally:
            if f:
                try: f.close()
                except StandardError, e: 
                    print e 
    else:
        print "No file mapping file found named: %s" % fname

def getDbName():
    """Determine database base name.

        Name is either (in this priority) read from
        - environment variable SSL_CLIENT_S_DN_CN
        - parameter DB_NAME_FROM_LIST and stored in session
        - session
    """
 
    dbname = None
    if request.environ.has_key('SSL_CLIENT_S_DN_CN'):
        try:
            dbname = request.environ['SSL_CLIENT_S_DN_CN'].split(' ')[3].lower()
            return NEW_DBS.get(dbname, dbname)
        except: 
            print >> sys.stderr, "Could not determine database name from client certificate."
    elif request.params.has_key('DB_NAME_FROM_LIST'):
        dbname = request.params['DB_NAME_FROM_LIST'].lower()
        session['DB_NAME'] = dbname
        session.save()
        return dbname
    elif session.has_key('DB_NAME'):
        return session['DB_NAME']
    else:
        print >> sys.stderr, "Could not determine database name"
    return dbname 

def checkRole(role):
    if type(role) in (ListType, TupleType): _role = role
    else:                                   _role = ( role, )

    def validate(func, self, *args, **kwargs):
        if not hasRole(_role):
            raise HTTPUnauthorized(detail = UNAUTHORIZED_DETAIL)
        try:
            return func(self, *args, **kwargs)
        except dbapi.OperationalError, err:
            raise HTTPUnauthorized(detail = UNAUTHORIZED_DETAIL)

    return decorator(validate)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
