# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

import sys

from paste.httpexceptions  import HTTPNotFound
from waskaweb.lib.helpers  import getHelp

NEXT_KEYS = {
    "0"   : "26",
    "26"  : "1",
    "1"   : "3",
    "2"   : "7",
    "3"   : "4",
    "4"   : "5",
    "5"   : "2",
    "6"   : "7",
    "7"   : "8",
    "8"   : "10",
    "9"   : "12",
    "10"  : "12",
    "11"  : "12",
    "12"  : "32",
    "13"  : "14",
    "14"  : "18",
    "18"  : "20",
    "19"  : "21",
    "20"  : "21",
    "21"  : "23",
    "22"  : "23",
    "23"  : "24",
    "24"  : "25",
    "27"  : "14",
    "28"  : "14",
    "29"  : "14",
    "30"  : "14",
    "31"  : "32",
    "32"  : "33",
    "33"  : "34",
    "34"  : "35",
    "35"  : "13",
    "25"  : "37",
    "36"  : "1"
}

class TreeItem:

    def __init__(self, key, page = "", description = "", children=None, enabled=True, fname=None):
        self.displayChildren = False
        self.key             = key
        self.formed_name     = fname
        self.page            = page
        self.parent          = None 
        self.description     = description
        self.children        = children
        self.enabled         = enabled 

    def __str__(self):
        return "Desc: %s, Key: %s, Fname: %s, Page: %s" % (self.description, self.key, self.formed_name, self.page)

    def toggle(self):
        self.displayChildren = not self.displayChildren

    def setParent(self, treeItem):
        self.parent = treeItem

    def hasEnabledChildren(self):
        """Return true/false depending if the item has enabled children."""
        if self.children and len(self.children) > 0:
            for child in self.children:
                if child.enabled: return True
        return False

    def hasDisplayedChildren(self):
        """Return true/false depending if item has children to be displayed."""
        if self.children and len(self.children) > 0:
            return self.displayChildren
        return False

    def render(self, select_action, toggle_action, out, 
                error=False, selected_key=None):
        err_str = ""
        if self.enabled:

            hl = self.key == selected_key and 'class="formpage_highlight"' or ""
            out.append('<li class="formpage"><span %s>' % hl)

            if self.hasEnabledChildren():
                if error:
                    symbol = '<img src="/images/icons/folder_closed_blue_2_error.gif" alt="Geschlossener Ordner Fehler" title="Ordner &ouml;ffnen - Fehler in Fallakte" border="0">'
                else: 
                    symbol = '<img src="/images/icons/folder_closed_blue_2.gif" alt="Geschlossener Ordner" title="Ordner &ouml;ffnen" border="0">'
            else:
                if error:
                    symbol = '<img src="/images/icons/formular_page_blue_error.gif" alt="Formular" title="Formular - Fehler enthalten" border="0">'
                else:
                    symbol = '<img src="/images/icons/formular_page_blue.gif" alt="Formular" title="Formular &ouml;ffnen" border="0">'

            out.append('<a href="%s" onclick="return checkModification();">%s</a>' % (
                toggle_action(self),
                symbol))

            out.append('<a href="%s" onclick="return checkModification();">%s%s</a>' % (
                select_action(self),
                err_str, 
                self.description))

            out.append('</span>')

            if self.hasDisplayedChildren():
                out.append('<ul>')
                for child in self.children:
                    child.render(
                        select_action, 
                        toggle_action, 
                        out, 
                        selected_key=selected_key
                    )
                out.append('</ul>')
            out.append("</li>")

    def realId(self, id):
        return id

    def nextKey(self):
        return NEXT_KEYS.get(self.key)

    def showPath(self):
        current = self
        while current:
            current.displayChildren = True
            current = current.parent

class KompetenzTreeItem(TreeItem):

    def __init__(self, key, page, description, children, kid, fname):
        TreeItem.__init__(self, key, page, description, children, fname=fname)
        self.kid = kid

    def realId(self, id):
        return self.kid

class NavigationTree:

    #def __init__(self, factories=None):
    def __init__(self, case=None):
        self.errors = [] #List page name with faulty pages
        factories = {}
        k_id = case.getKompetenzId()
        
        def xmake(key, page, description, children=None, fname=None):
            return KompetenzTreeItem(key, page, description, children, k_id, fname)

        if not k_id is None:
            factories = {
            "page_8": xmake,
            "page_9": xmake,
            "page_26": xmake,
            "page_27": xmake,
            "page_28": xmake,
            "page_29": xmake
            }

        def make(p):
            def _make(*args, **v):
                return factories.get(p, TreeItem)(*args, **v)
            return _make

        self.children = [
            make("page_0")("0", "page_0", "Zust&auml;ndigkeiten, Eingangsdaten", fname="info-0"),
            make("page_25")("26", "page_25", "Externe Ansprechpartner", fname="info-28"),
            make("page_1")("36", "page_1", "Angaben zur Person", [
                make("page_1")("1", "page_1", "Pers&ouml;nliche Angaben"),
                make("page_2")("3", "page_2", "Famili&auml;re Situation"),
                make("page_3")("4", "page_3", "Gesundheitliche Situation"),
                make("page_4")("5", "page_4", "Soziale Lage")
            ], fname="info-1"),
            make("page_2")("2", "page_5", "Biographische Daten", [
                make("page_5")("6", "page_5", "Schule"),
                make("page_6")("7", "page_6", "Ausbildung und Arbeit"),
                make("page_7")("8", "page_7", "Weiterer Verlauf", fname="info-7")
            ], fname="info-2"),
            make("page_8")("9", "page_8", "Case-Management", [
                make("page_8")("10", "page_8", "Kompetenzfeststellung", [
                    make("page_8")("11", "page_8", "Verfahren"),
                    make("page_9")("12", "page_9", "Ergebnisse der Kompetenzfeststellung", [
                        make("page_9")("31", "page_9", "Fachkompetenzen"),
                        make("page_26")("32", "page_26", "Methodenkompetenzen"),
                        make("page_27")("33", "page_27", "Sprachen und Kommunikation"),
                        make("page_28")("34", "page_28", "Sozial und Selbstkompetenzen")
                    ], fname="info-9"),
                    make("page_29")("35", "page_29", "Dokumentation")
                ]),
                make("page_10")("13", "page_10", "F&ouml;rderplanung", fname="info-14"),
                make("rg_unterstuetzungsangebote")("14", "rg_unterstuetzungsangebote", "Unterst&uuml;tzungsangebote", [
                    make("page_21")("27", "page_21", "Angebot im Bereich BB", None, False),
                    make("page_22")("28", "page_22", "Angebot im Bereich BV", None, False),
                    make("page_23")("29", "page_23", "Angebot im Bereich BQ", None, False),
                    make("page_24")("30", "page_24", "Angebot im Bereich LB", None, False)
                ]),
                make("page_15")("18", "page_15", "Freie Dokumentation")
            ], fname="info-8"),
            make("page_16")("19", "page_16", "Abschlussbewertung", [
                make("page_16")("20", "page_16", "Angaben zum Verlauf und Beendigung des CM"),
                make("page_17")("21", "page_17", "Erreichte Oberziele", [
                    make("page_17")("22", "page_17", "Berufliche Integration"),
                    make("page_18")("23", "page_18", "Soziale Integration"),
                    make("page_19")("24", "page_19", "Einsch&auml;tzung des Case-Managers")
                ]),
                make("page_20")("25", "page_20", "Freie Dokumentation")
            ], fname="info-20"),
            make("page_nachbetreuung")("37", "page_nachbetreuung", "Nachbetreuung")
        ]
        self.buildParents(self.children)

    def buildParents(self, children, parent=None):
        for child in children:
            if parent:
                child.setParent(parent)
            if child.children: 
                self.buildParents(child.children, child)

    def getAllItems(self, node=None):
        '''Returns a serialized list of treeitems'''
        if not node: node = self
        items = []
        for c in node.children:
            items.append(c)
            if c.children:
                items.extend(self.getAllItems(c))
        return items

    def getTreeItem(self, key):
        stack = [self.children]
        while stack:
            children = stack.pop()
            for c in children:
                if c.key == str(key):
                    return c
                if c.children:
                    stack.append(c.children)
        return None

    def getHeaders(self, key):
        headers = []
        ti = self.getTreeItem(key)
        if not ti: return headers
        # we need to look forward in the children to get page which will be
        # actually rendered
        ti = self.getTreeItemByPageName(ti.page)
        while True:
            headers.append((ti.description, getHelp(ti.formed_name)))
            if not ti.parent: break
            ti = ti.parent
        headers.reverse()
        return headers

    def findTreeItemByPageName(self, name):
        stack = [self.children]
        item = None
        while stack:
            children = stack.pop()
            for c in children:
                if c.page == name:
                    return c
                if c.children:
                    stack.append(c.children)
        return None

    def getTreeItemByPageName(self, name):
        '''Returns the treeitem which will be actually rendered when clicking
        on the item in navigation tree. This is the last treeitem in the tree
        with the given name'''
        item = self.findTreeItemByPageName(name)
        if not item: raise HTTPNotFound()
        return item

    def render(self, select_action, toggle_action, selected_key=None):
        out = ['<ul>']
        for child in self.children:
            child.render(select_action, toggle_action, out, (child.page in self.errors), selected_key)
        out.append('</ul>')
        return "\n".join(out)

    def setErrors(self, errors):
        self.errors = errors



# vim:set ts=4 sw=4 si et sta sts=4:
