# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
# Authors:
#
# Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from waskaweb.model.phase_transition import \
    phase_neighbors, phase_symbol, phase_description, phase_predecessors_path, phase_successors_path

class Phase:

    def __init__(self, phase, running = False, complete = False, time = None, links = ""):
        self.phase    = phase
        self.running  = running
        self.complete = complete
        self.time     = time
        self.links    = links
        self.preds    = []

    def isNeighbor(self, other):
        return phase_neighbors(self.phase, other.phase)

    def isRunning(self):
        return self.running

    def addPredecessor(self, pred):
        self.preds.append(pred)

    def getLabel(self):
        return phase_symbol(self.phase)

    def getDescription(self):
        return phase_description(self.phase)

    def isComplete(self):
        return self.complete

    def isRecursiveComplete(self):
        if not self.complete:
            return False

        for pred in self.preds:
            if not pred.isRecursiveComplete():
                return False

        return True

    def hasPredecessor(self, other):
        #for p in self.preds:
        #    if other.phase == p.phase or p.hasPredecessor(other): return True
        #return False
        predecessors = phase_predecessors_path(self.phase)
        return other.phase in predecessors

    def hasSuccessor(self, other):
        successors = phase_successors_path(self.phase)
        return other.phase in successors

    def getTime(self):
        return self.time

    def getLinks(self):
        return self.links

class PhasePair:

    def __init__(self, start, end, description):
        self.start       = start
        self.end         = end
        self.description = description

    def getStart(self):
        return self.start

    def getEnd(self):
        return self.end

    def getDuration(self):
        t1 = self.start.getTime()
        t2 = self.end.getTime()
        if not t1 or not t2: return None
        return t2 - t1

    def getDescription(self):
        return self.description

    def isRunning(self):
        return self.start.isRunning() or self.end.isRunning()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
